# Suppress Output from IPhython Notebooks for git

Taken from https://gist.github.com/pbugnion/ea2797393033b54674af

Suppress output and prompt numbers in git version control.
This script will tell git to ignore prompt numbers and cell output
when looking at `ipynb` files if their metadata contains:

    ```
    "git" : { "suppress_output" : true }
    ```

The notebooks themselves are not changed.
See also [this blogpost](http://pascalbugnion.net/blog/ipython-notebooks-and-git.html).

## Usage instructions

1. Put this script in a directory that is on the system's path.
   For future reference, I will assume you left it in this repository:

```shell
</path/to/rdstar/repo/>git_ipython_drop_output/ipynb_drop_output.py.
```

2. Make sure it is executable by typing the command

```shell
chmod +x </path/to/rdstar/repo/>git_ipython_drop_output/ipynb_drop_output.py.
```

3. Register a filter for ipython notebooks by putting the following line
in the `.gitattributes` file of this repository:

```
*.ipynb  filter=clean_ipynb
```

You can also add this to `.git/info/attributes` in this repository,
if you do not want to commit this or further changes to the attributes.

4. Connect this script to the filter by adding the following git commands:

```shell
git config --local filter.clean_ipynb.clean </path/to/rdstar/repo/>git_ipython_drop_output/ipynb_drop_output.py
git config --local filter.clean_ipynb.smudge cat
```

To tell git to ignore the output and prompts for a notebook,
open the notebook's metadata (Edit > Edit Notebook Metadata).
A panel should open containing the lines:

```
{
    "name" : "",
    "signature" : "some very long hash"
}
```

Add an extra line so that the metadata now looks like:

```
{
    "name" : "",
    "signature" : "don't change the hash, but add a comma at the end of the line",
    "git" : { "suppress_outputs" : true }
}
```

This can also be done by editing the `ipynb` files manually, e.g. via `vim`.

You may need to "touch" the notebooks for git to actually register a change,
if your notebooks are already under version control.

To quickly touch and add all `ipynb` files use the commands:

```shell
find ./ -type f -name "*.ipynb" -exec touch {} +
find ./ -type f -name "*.ipynb" -exec git add {} +
```

## Use this Filter Globally

To use this filter globally, just but the script into an appropriate place, e.g.

```shell
/home/scripts/ipynb_drop_output.py.
```

add the changes to the `attributes` file to `~/.git/attributes` instead of the local file `.gitattributes`,
which is only valid for this repository, and finally add the filter globally instead of locally:

```shell
git config --local filter.clean_ipynb.clean /home/scripts/ipynb_drop_output.py
git config --local filter.clean_ipynb.smudge cat
```

## Notes

This script is inspired by http://stackoverflow.com/a/20844506/827862, but
lets the user specify whether the output of a notebook should be suppressed
in the notebook's metadata, and works for IPython v3.0.
