"""
Providing latex strings for names of observables.

Felix Metzner 2021
"""

from typing import List

from rdstar.latex_labels.latex_particle_names import LatexParticleNames
from rdstar.latex_labels.latex_string_methods import LatexObsNameMethods

__all__ = [
    "LatexObsNames",
]


class LatexObsNames:
    # fmt: off
    mother_str = r"$\mathrm{Mother}$"  # type: str
    daughter_str = r"$\mathrm{Daughter}$"  # type: str
    first_daughter_str = r"$1^\mathrm{st}\;" + daughter_str[1:]

    __miss_str = r"miss"  # type: str
    miss_str = r"$\mathrm{" + __miss_str + r"}$"  # type: str
    miss_sig_str = r"$\mathrm{" + __miss_str + r",\;sig}$"  # type: str
    miss_tag_str = r"$\mathrm{" + __miss_str + r",\;tag}$"  # type: str

    star_str = r"$\ast$"  # type: str

    reco_tag = r"$\mathrm{Reco}$"  # type: str
    mc_tag = r"$\mathrm{MC}$"  # type: str
    truth_tag = r"$\mathrm{Truth}$"  # type: str

    lab_tag = r"$\mathrm{Lab}$"  # type: str
    cms_tag = r"$\mathrm{CMS}$"  # type: str
    rest_tag = r"$\mathrm{Rest}$"  # type: str

    fei_str = r"$\mathrm{FEI}$"  # type: str

    mc_match_sig_id = r"$\mathrm{Signal\;ID}$"  # type: str
    mc_match_gen_id = r"$\mathrm{Generator\;ID}$"  # type: str
    mc_info_data_type = r"$\mathrm{MC\;Type}$"  # type: str
    data_type = r"$\mathrm{Data\;Type}$"  # type: str

    reco_pdg_code = r"$\mathrm{PDG\;ID}$"  # type: str

    reco_mode_id = r"$\mathrm{Reco\;Mode\;ID}$"  # type: str
    reco_d_reco_mode_id = LatexParticleNames.d_str[:-1] + r"\;" + reco_mode_id[1:]  # type: str
    reco_b_reco_mode_id = LatexParticleNames.b_str[:-1] + r"\;" + reco_mode_id[1:]  # type: str
    reco_bsig_reco_mode_id = LatexParticleNames.b_sig_str[:-1] + r"\;" + reco_mode_id[1:]  # type: str
    reco_b_tag_reco_mode_id = LatexParticleNames.b_tag_str[:-1] + r"\;" + reco_mode_id[1:]  # type: str
    reco_y4s_reco_mode_id = LatexParticleNames.y4s_str[:-1] + r"\;" + reco_mode_id[1:]  # type: str

    reco_decay_mode_id = r"$\mathrm{Decay\;Mode\;ID}$"  # type: str
    reco_d_decay_mode_id = LatexParticleNames.d_str[:-1] + r"\;" + reco_decay_mode_id[1:]  # type: str
    reco_b_decay_mode_id = LatexParticleNames.b_str[:-1] + r"\;" + reco_decay_mode_id[1:]  # type: str
    reco_bsig_decay_mode_id = LatexParticleNames.b_sig_str[:-1] + r"\;" + reco_decay_mode_id[1:]  # type: str
    reco_b_tag_decay_mode_id = LatexParticleNames.b_tag_str[:-1] + r"\;" + reco_decay_mode_id[1:]  # type: str
    reco_y4s_decay_mode_id = LatexParticleNames.y4s_str[:-1] + r"\;" + reco_decay_mode_id[1:]  # type: str

    reco_mode_label = r"$\mathrm{Reco\;Mode}$"  # type: str
    reco_y4s_reco_mode_label = LatexParticleNames.y4s_str[:-1] + r"\;" + reco_mode_label[1:]  # type: str
    reco_b_sig_reco_mode_label = LatexParticleNames.b_sig_str[:-1] + r"\;" + reco_mode_label[1:]  # type: str
    reco_b_tag_reco_mode_label = LatexParticleNames.b_tag_str[:-1] + r"\;" + reco_mode_label[1:]  # type: str
    reco_d_reco_mode_label = LatexParticleNames.d_str[:-1] + r"\;" + reco_mode_label[1:]  # type: str
    reco_d_or_dst_reco_mode_label = LatexParticleNames.d_w_wo_st_str[:-1] + r"\;" + reco_mode_label[1:]  # type: str

    reco_decay_mode_label = r"$\mathrm{Decay\;Mode}$"  # type: str
    reco_y4s_decay_mode_label = LatexParticleNames.y4s_str[:-1] + r"\;" + reco_decay_mode_label[1:]  # type: str
    reco_b_sig_decay_mode_label = LatexParticleNames.b_sig_str[:-1] + r"\;" + reco_decay_mode_label[1:]  # type: str
    reco_b_tag_decay_mode_label = LatexParticleNames.b_tag_str[:-1] + r"\;" + reco_decay_mode_label[1:]  # type: str
    reco_d_decay_mode_label = LatexParticleNames.d_str[:-1] + r"\;" + reco_decay_mode_label[1:]  # type: str
    reco_d_or_dst_decay_mode_label = LatexParticleNames.d_w_wo_st_str[:-1] + r"\;" + reco_decay_mode_label[1:]  # type: str

    momentum_p = r"$\mathrm{p}$"  # type: str
    momentum_pt = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\mathrm{t}$")  # type: str
    momentum_px = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\mathrm{x}$")  # type: str
    momentum_py = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\mathrm{y}$")  # type: str
    momentum_pz = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\mathrm{z}$")  # type: str
    momentum_theta = r"$\theta$"  # type: str
    momentum_cos_theta = r"$\cos(\theta)$"  # type: str
    momentum_phi = r"$\phi$"  # type: str
    momentum_p_cms = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=cms_tag)  # type: str
    momentum_pt_cms = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\mathrm{t},\;" + cms_tag[1:])  # type: str

    energy_e = r"$\mathrm{E}$"  # type: str
    energy_delta_e = LatexObsNameMethods.get_delta_of(var_str=energy_e)  # type: str
    energy_q = r"$\mathrm{Q}$"  # type: str
    energy_delta_q = LatexObsNameMethods.get_delta_of(var_str=energy_q)  # type: str

    mass_m = r"$\mathrm{M}$"  # type: str
    mass_delta_m = LatexObsNameMethods.get_delta_of(var_str=mass_m)  # type: str
    mass_abs_delta_m = LatexObsNameMethods.get_abs_of(var_str=mass_delta_m)  # type: str
    mass_inv_m = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=r"$\mathrm{Inv}$")  # type: str
    mass_sig_mbf = mass_delta_m[:-1] + LatexObsNameMethods.add_sub_script(var_str=r"$ / \sigma$", script=mass_m)[1:]  # type: str
    mass_abs_sig_mbf = LatexObsNameMethods.get_abs_of(var_str=mass_sig_mbf)  # type: str
    mass_delta_m_1st_daughter_mother = LatexObsNameMethods.add_sub_script(var_str=mass_delta_m, script=first_daughter_str[:-1] + r"-" + mother_str[1:])  # type: str  # noqa: E501, E950
    mass_sigma_delta_m_1st_daughter_mother = LatexObsNameMethods.create_x_of_y(x=r"$\sigma$", y=mass_delta_m_1st_daughter_mother)

    impact_delta_r = LatexObsNameMethods.get_delta_of(var_str=r"$\mathrm{r}$")  # type: str
    impact_delta_z = LatexObsNameMethods.get_delta_of(var_str=r"$\mathrm{z}$")  # type: str
    impact_abs_delta_z = LatexObsNameMethods.get_abs_of(var_str=impact_delta_z)  # type: str

    tracks_p_value = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{Track\;Fit}$")  # type: str
    tracks_n_cdc_hits = r"$\mathrm{N}_\mathrm{CDC\;Hits}^\mathrm{Track}$"  # type: str
    tracks_n_svd_hits = r"$\mathrm{N}_\mathrm{SVD\;Hits}^\mathrm{Track}$"  # type: str
    tracks_curler_clone_indicator = r"$\mathrm{Clone\;from\;Curler\;Indicator}$"  # type: str

    cluster_good_belle_gamma = r"$\mathrm{Good\;Belle\;Gamma\;Indicator}$"  # type: str
    cluster_e1_over_e9 = r"$\mathrm{E}1 / \mathrm{E}9$"  # type: str
    cluster_e9_over_e21 = r"$\mathrm{E}9 / \mathrm{E}21$"  # type: str
    cluster_e9_over_e25 = r"$\mathrm{E}9 / \mathrm{E}25$"  # type: str
    cluster_n_hits = r"$\mathrm{N}_\mathrm{ECL\;Hits}^\mathrm{Cluster}$"  # type: str
    cluster_highest_e = r"$\mathrm{E}_\mathrm{max}^\mathrm{Cluster}$"  # type: str

    pid_electron = r"$\mathrm{eID}$"  # type: str
    pid_muon = r"$\mu\mathrm{ID}$"  # type: str
    pid_muon_quality = r"$\mu\;\mathrm{Quality}$"  # type: str
    pid_atc23 = r"$\pi^\pm\mathrm{ID}$"  # type: str
    pid_atc32 = r"K$^\pm\mathrm{ID}$"  # type: str
    pid_ksnb_standard = LatexObsNameMethods.get_probability_of(sub=LatexParticleNames.k_short_str, sup=r"$\mathrm{NB\;std}$")  # type: str
    pid_good_ks = r"$\mathrm{Good\;}" + LatexParticleNames.k_short_str[1:-1] + r"\mathrm{\;Indicator}$"  # type: str

    roe_e_ecl_extra = r"$\mathrm{E}^\mathrm{ECL}_\mathrm{extra}$"  # type: str
    roe_n_e_ecl_extra = r"$\mathrm{nE}^\mathrm{ECL}_\mathrm{extra}$"  # type: str
    roe_n_tracks = r"$n_\mathrm{Tracks}^\mathrm{ROE}$"  # type: str

    ecl_belle_energy = r"$\mathrm{E}^\mathrm{ECL}_\mathrm{Belle}$"  # type: str

    fei_prob = LatexObsNameMethods.get_probability_of(sub=fei_str)  # type: str

    _ff_cos_theta = r"$\cos \theta$"  # type: str
    ff_w = r"$\mathrm{w}$"  # type: str
    ff_chi = r"$\chi$"  # type: str
    ff_chi_prime = r"$\chi{}'$"  # type: str
    ff_cos_theta_l = LatexObsNameMethods.add_sub_script(var_str=_ff_cos_theta, script=LatexParticleNames.lepton_str)  # type: str
    ff_cos_theta_nu = r"$\cos \theta_\nu$"  # type: str
    ff_w_reco = LatexObsNameMethods.add_sub_script(var_str=ff_w, script=reco_tag)  # type: str
    ff_chi_reco = LatexObsNameMethods.add_sub_script(var_str=ff_chi, script=reco_tag)  # type: str
    ff_chi_prime_reco = LatexObsNameMethods.add_sub_script(var_str=ff_chi_prime, script=reco_tag)  # type: str
    ff_cos_theta_l_reco = _ff_cos_theta[:-1] + r"_{" + LatexParticleNames.lepton_str[1:-1] + ",\;" + reco_tag[1:-1] + r"}$"  # type: str  # noqa: W605
    ff_cos_theta_nu_reco = _ff_cos_theta[:-1] + r"_{\nu,\;" + reco_tag[1:-1] + r"}$"  # type: str
    ff_w_truth = LatexObsNameMethods.add_sub_script(var_str=ff_w, script=truth_tag)  # type: str

    lepton_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=LatexParticleNames.lepton_str)  # type: str
    lepton_p_sup = LatexObsNameMethods.add_super_script(var_str=momentum_p, script=LatexParticleNames.lepton_str)  # type: str
    lepton_cms_p = LatexObsNameMethods.add_sub_script(var_str=lepton_p_sup, script=cms_tag)  # type: str
    lepton_cms_p_t = LatexObsNameMethods.add_sub_script(var_str=lepton_p_sup, script=r"$\mathrm{t,\;" + cms_tag[1:-1] + "}$")  # type: str

    lepton_theta = LatexObsNameMethods.add_sub_script(var_str=momentum_theta, script=LatexParticleNames.lepton_str)  # type: str

    _lepton_bin = LatexObsNameMethods.add_super_script(var_str=r"$\mathrm{Bin}$", script=LatexParticleNames.lepton_str)  # type: str
    lepton_p_bin = LatexObsNameMethods.add_sub_script(var_str=_lepton_bin, script=momentum_p)  # type: str
    lepton_theta_bin = LatexObsNameMethods.add_sub_script(var_str=_lepton_bin, script=momentum_theta)  # type: str
    _bin_momentum_theta = momentum_p[:-1] + r",\," + momentum_theta[1:]  # type: str
    lepton_p_theta_bin = LatexObsNameMethods.add_sub_script(var_str=_lepton_bin, script=_bin_momentum_theta)  # type: str

    lepton_pdg_id = LatexObsNameMethods.add_sub_script(var_str=reco_pdg_code, script=LatexParticleNames.lepton_str)  # type: str

    d_meson_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=LatexParticleNames.d_str)  # type: str
    d_star_meson_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=LatexParticleNames.d_st_str)  # type: str
    d_w_wo_star_meson_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=LatexParticleNames.d_w_wo_st_str)  # type: str

    d_st_daughter_str = LatexParticleNames.d_st_str[:-1] + daughter_str[1:]  # type: str
    d_star_daughter_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=d_st_daughter_str)  # type: str
    d_w_wo_st_daughter_str = LatexParticleNames.d_w_wo_st_str[:-1] + daughter_str[1:]  # type: str
    d_w_wo_star_daughter_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=d_w_wo_st_daughter_str)  # type: str

    d_w_wo_st_mass = LatexObsNameMethods.create_x_of_y(x=mass_m, y=LatexParticleNames.d_w_wo_st_str)  # type: str
    d_w_wo_st_delta_m = LatexObsNameMethods.add_sub_script(var_str=mass_delta_m, script=LatexParticleNames.d_w_wo_st_str)  # type: str
    d_w_wo_st_abs_delta_m = LatexObsNameMethods.get_abs_of(var_str=d_w_wo_st_delta_m)  # type: str
    d_w_wo_st_and_pi_mass = LatexObsNameMethods.create_x_of_y(x=mass_m, y=LatexParticleNames.d_w_wo_st_and_pi_str)  # type: str

    d_w_wo_st_pdg_id = LatexObsNameMethods.add_sub_script(var_str=reco_pdg_code, script=LatexParticleNames.d_w_wo_st_and_pi_str)  # type: str

    d_d_st_delta_m = LatexObsNameMethods.add_sub_script(
        var_str=mass_delta_m,
        script=LatexParticleNames.d_st_str[:-1] + ",\," + LatexParticleNames.d_str[1:]  # noqa: W605
    )  # type: str
    d_st_st_delta_m = d_w_wo_st_and_pi_mass[:-1] + r" - " + d_w_wo_st_mass[1:]  # type: str
    d_st_daughters_inv_m = mass_inv_m + r"$\,_{" + LatexParticleNames.d_st_daughters_str[1:-1] + "}$"  # type: str

    d_st_daughters_angle = LatexObsNameMethods.create_angle_of_x(x=LatexParticleNames.d_st_daughters_str)  # type: str
    d_st_daughters_lab_angle = LatexObsNameMethods.add_super_script(var_str=d_st_daughters_angle, script=lab_tag)  # type: str
    d_st_daughters_lab_cos_angle = LatexObsNameMethods.create_x_of_y(x=r"\cos", y=d_st_daughters_lab_angle)  # type: str

    _from_dst = r"$\mathrm{\;from\;}" + LatexParticleNames.d_st_str[1:]
    d_st_meson_pi_cms_p = LatexObsNameMethods.add_super_script(var_str=momentum_p_cms, script=r"$\pi" + _from_dst[1:])  # type: str

    _from_dwwost = r"$\mathrm{\;from\;}" + LatexParticleNames.d_w_wo_st_str[1:]
    d_st_meson_pi_1_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$,\;\pi_1" + _from_dwwost[1:])  # type: str
    d_st_meson_pi_2_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$,\;\pi_2" + _from_dwwost[1:])  # type: str
    d_st_meson_pi_3_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$,\;\pi_3" + _from_dwwost[1:])  # type: str
    d_st_meson_pi_1_cms_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=cms_tag[:-1] + r",\;\pi\,1" + _from_dwwost[1:])  # type: str  # noqa: E501, E950
    d_st_meson_pi_2_cms_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=cms_tag[:-1] + r",\;\pi\,2" + _from_dwwost[1:])  # type: str  # noqa: E501, E950
    d_st_meson_pi_3_cms_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=cms_tag[:-1] + r",\;\pi\,3" + _from_dwwost[1:])  # type: str  # noqa: E501, E950

    _from_dstst = r"$\mathrm{\;from\;}" + LatexParticleNames.d_st_st_str[1:]
    d_st_st_meson_pi0_cms_p = LatexObsNameMethods.add_super_script(var_str=momentum_p_cms, script=r"$\pi^0" + _from_dstst[1:])  # type: str
    d_st_st_meson_pi_cms_p = LatexObsNameMethods.add_super_script(var_str=momentum_p_cms, script=r"$\pi" + _from_dstst[1:])  # type: str
    d_st_st_meson_pi0_m_inv = LatexObsNameMethods.add_super_script(var_str=mass_inv_m, script=r"$\pi^0" + _from_dstst[1:])  # type: str

    __b_mbc = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=r"$\mathrm{bc}$")  # type: str

    b_sig_q = LatexObsNameMethods.add_sub_script(var_str=energy_q, script=LatexParticleNames.b_sig_str)  # type: str
    b_sig_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=LatexParticleNames.b_sig_str)  # type: str
    b_sig_vertex_prob = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{Vertex}$", sup=LatexParticleNames.b_sig_str)  # type: str

    b_tag_mbc = LatexObsNameMethods.add_super_script(var_str=__b_mbc, script=LatexParticleNames.tag_str)  # type: str
    b_tag_delta_e = LatexObsNameMethods.add_sub_script(var_str=energy_delta_e, script=LatexParticleNames.tag_str)  # type: str
    b_tag_vertex_prob = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{Vertex}$", sup=LatexParticleNames.b_tag_str)  # type: str
    b_tag_daughter_vertex_prob = LatexObsNameMethods.get_probability_of(
        sub=r"$\mathrm{Vertex}$",
        sup=LatexParticleNames.b_tag_str[:-1] + r"\;\mathrm{Daughter}$"
    )  # type: str

    y4s_momentum = LatexObsNameMethods.add_super_script(var_str=momentum_p, script=LatexParticleNames.y4s_str)  # type: str
    y4s_cms_momentum = LatexObsNameMethods.add_sub_script(var_str=y4s_momentum, script=cms_tag)  # type: str
    y4s_mass = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=LatexParticleNames.y4s_str)  # type: str

    __m_sq = LatexObsNameMethods.add_super_script(var_str=mass_m, script=r"$2$")  # type: str
    fitting_p_l_star = LatexObsNameMethods.add_super_script(var_str=lepton_p, script=star_str)  # type: str
    fitting_mm2 = LatexObsNameMethods.add_sub_script(var_str=__m_sq, script=miss_str)  # type: str
    fitting_q2 = r"$\mathrm{q}^2$"  # type: str

    fitting_e_miss = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=miss_str)  # type: str
    fitting_p_miss = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=miss_str)  # type: str
    fitting_e_miss_cms = LatexObsNameMethods.add_super_script(var_str=fitting_e_miss, script=cms_tag)  # type: str
    fitting_p_miss_cms = LatexObsNameMethods.add_super_script(var_str=fitting_p_miss, script=cms_tag)  # type: str
    fitting_p_miss_star = LatexObsNameMethods.add_super_script(var_str=fitting_p_miss, script=star_str)  # type: str
    fitting_e_miss_minus_p_miss = fitting_e_miss[:-1] + r" - " + fitting_p_miss[1:]  # type: str
    fitting_e_miss_minus_p_miss_cms = fitting_e_miss_cms[:-1] + r" - " + fitting_p_miss_cms[1:]  # type: str

    mva_cs_output = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{CS}$")  # type: str
    mva_sel_output = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{Selection\;MVA}$")  # type: str
    mva_sel_output_short = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{Sel}$")  # type: str
    mva_default_output = LatexObsNameMethods.get_probability_of(sub=r"$\mathrm{MVA}$")  # type: str
    mva_mm2_sig = LatexObsNameMethods.add_sub_script(var_str=__m_sq, script=r"$\mathrm{miss,\;sig}$")  # type: str
    mva_mm2_bkg = LatexObsNameMethods.add_sub_script(var_str=__m_sq, script=r"$\mathrm{miss,\;bkg}$")  # type: str

    cs_r2 = r"$\mathrm{R}_2$"  # type: str

    __fsr_str = r"$\mathrm{FSR}$"  # type: str
    fsr_gamma_str = __fsr_str[:-1] + r"\;" + LatexParticleNames.gamma_str[1:]  # type: str
    fsr_el_gamma_str = LatexParticleNames.electron_str[:-1] + LatexParticleNames.gamma_str[1:]  # type: str
    fsr_energy = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=__fsr_str)  # type: str
    fsr_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=__fsr_str)  # type: str
    fsr_cms_energy = LatexObsNameMethods.add_super_script(var_str=fsr_energy, script=cms_tag)  # type: str
    fsr_rest_energy = LatexObsNameMethods.add_super_script(var_str=fsr_energy, script=rest_tag)  # type: str
    fsr_lab_energy = LatexObsNameMethods.add_super_script(var_str=fsr_energy, script=lab_tag)  # type: str
    fsr_cms_p = LatexObsNameMethods.add_super_script(var_str=fsr_p, script=cms_tag)  # type: str
    fsr_rest_p = LatexObsNameMethods.add_super_script(var_str=fsr_p, script=rest_tag)  # type: str
    fsr_lab_p = LatexObsNameMethods.add_super_script(var_str=fsr_p, script=lab_tag)  # type: str
    fsr_e_truth = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=mc_tag)  # type: str
    fsr_p_truth = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=mc_tag)  # type: str
    fsr_e_reco = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=reco_tag)  # type: str
    fsr_p_reco = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=reco_tag)  # type: str
    fsr_e_res = fsr_e_reco[:-1] + r" - " + fsr_e_truth[1:]  # type: str
    fsr_p_res = fsr_p_reco[:-1] + r" - " + fsr_p_truth[1:]  # type: str
    fsr_pt = LatexObsNameMethods.add_super_script(var_str=momentum_pt, script=__fsr_str)  # type: str
    fsr_px = LatexObsNameMethods.add_super_script(var_str=momentum_px, script=__fsr_str)  # type: str
    fsr_py = LatexObsNameMethods.add_super_script(var_str=momentum_py, script=__fsr_str)  # type: str
    fsr_pz = LatexObsNameMethods.add_super_script(var_str=momentum_pz, script=__fsr_str)  # type: str
    fsr_gamma_energy = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=fsr_gamma_str)  # type: str
    fsr_gamma_p = LatexObsNameMethods.add_super_script(var_str=momentum_pt, script=fsr_gamma_str)  # type: str
    fsr_gamma_px = LatexObsNameMethods.add_super_script(var_str=momentum_px, script=fsr_gamma_str)  # type: str
    fsr_gamma_py = LatexObsNameMethods.add_super_script(var_str=momentum_py, script=fsr_gamma_str)  # type: str
    fsr_gamma_pz = LatexObsNameMethods.add_super_script(var_str=momentum_pz, script=fsr_gamma_str)  # type: str
    fsr_corrected_e_inv_m = LatexObsNameMethods.add_super_script(var_str=mass_inv_m, script=fsr_el_gamma_str)  # type: str
    fsr_corrected_e = r"$\mathrm{FSR\;Corrected}\;" + LatexParticleNames.electron_str[1:]  # type: str
    fsr_photon_used_for_fsr = LatexParticleNames.gamma_str[:-1] + r"\;\mathrm{used\;by\;FSR\;Correction}$"  # type: str
    fsr_e_with_fsr = LatexParticleNames.electron_str[:-1] + r"\;\mathrm{with\;FSR\;Correction}$"  # type: str

    q2_truth = LatexObsNameMethods.add_sub_script(var_str=fitting_q2, script=truth_tag)  # type: str
    q2_reco = LatexObsNameMethods.add_sub_script(var_str=fitting_q2, script=reco_tag)  # type: str
    q2_reco_v2 = LatexObsNameMethods.add_sub_script(var_str=fitting_q2, script=reco_tag[:-1] + r"\;\mathrm{V2}$")  # type: str
    q2_reco_v3 = LatexObsNameMethods.add_sub_script(var_str=fitting_q2, script=reco_tag[:-1] + r"\;\mathrm{V3}$")  # type: str
    q2_reco_v4 = LatexObsNameMethods.add_sub_script(var_str=fitting_q2, script=reco_tag[:-1] + r"\;\mathrm{V4}$")  # type: str
    q2_resi = q2_reco[:-1] + r" - " + q2_truth[1:]  # type: str
    q2_resi_v2 = q2_reco_v2[:-1] + r" - " + q2_truth[1:]  # type: str
    q2_resi_v3 = q2_reco_v3[:-1] + r" - " + q2_truth[1:]  # type: str
    q2_resi_v4 = q2_reco_v4[:-1] + r" - " + q2_truth[1:]  # type: str
    q2_reso = q2_resi[:-1] + r" / " + q2_truth[1:]  # type: str
    q2_reso_v2 = q2_resi_v2[:-1] + r" / " + q2_truth[1:]  # type: str
    q2_reso_v3 = q2_resi_v3[:-1] + r" / " + q2_truth[1:]  # type: str
    q2_reso_v4 = q2_resi_v4[:-1] + r" / " + q2_truth[1:]  # type: str

    pi_momentum = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=r"$\pi$")  # type: str
    pi0_e = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=r"$\pi^0$")  # type: str
    pi0_m = LatexObsNameMethods.add_sub_script(var_str=mass_m, script=r"$\pi^0$")  # type: str
    pi0_delta_m = LatexObsNameMethods.add_sub_script(var_str=mass_delta_m, script=r"$\pi^0$")  # type: str
    pi0_abs_delta_m = LatexObsNameMethods.get_abs_of(var_str=pi0_delta_m)  # type: str
    pi0_good_gamma_0 = cluster_good_belle_gamma[:-1] + r"\;(" + LatexParticleNames.gamma_str[1:-1] + r"_1)$"  # type: str
    pi0_good_gamma_1 = cluster_good_belle_gamma[:-1] + r"\;(" + LatexParticleNames.gamma_str[1:-1] + r"_2)$"  # type: str
    pi0_sig_mbf = pi0_delta_m[:-1] + LatexObsNameMethods.add_sub_script(var_str=r"$ / \sigma$", script=pi0_m)[1:]  # type: str
    pi0_abs_sig_mbf = LatexObsNameMethods.get_abs_of(var_str=pi0_sig_mbf)  # type: str
    pi0_daughters_inv_m = mass_inv_m + r"$\,_{" + LatexParticleNames.gamma_gamma_str[1:-1] + "}$"  # type: str
    pi0_daughters_angle = LatexObsNameMethods.create_angle_of_x(x=LatexParticleNames.gamma_gamma_str)  # type: str
    pi0_daughters_cos_angle = LatexObsNameMethods.create_x_of_y(x=r"\cos", y=pi0_daughters_angle)  # type: str
    pi0_daughters_cms_angle = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_angle, script=cms_tag)  # type: str
    pi0_daughters_cms_cos_angle = LatexObsNameMethods.create_x_of_y(x=r"\cos", y=pi0_daughters_cms_angle)  # type: str
    pi0_daughters_rest_angle = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_angle, script=rest_tag)  # type: str
    pi0_daughters_rest_cos_angle = LatexObsNameMethods.create_x_of_y(x=r"\cos", y=pi0_daughters_rest_angle)  # type: str
    pi0_daughters_phi = LatexObsNameMethods.create_angle_of_x(x=LatexParticleNames.gamma_gamma_str, angle_symbol=r"$\phi$")  # type: str
    pi0_daughters_cms_phi = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_phi, script=cms_tag)  # type: str
    pi0_daughters_delta_phi = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_phi)  # type: str
    pi0_daughters_cms_delta_phi = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_cms_phi)  # type: str
    pi0_daughters_theta = LatexObsNameMethods.create_angle_of_x(x=LatexParticleNames.gamma_gamma_str, angle_symbol=r"$\theta$")  # type: str
    pi0_daughters_cms_theta = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_theta, script=cms_tag)  # type: str
    pi0_daughters_delta_theta = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_theta)  # type: str
    pi0_daughters_cms_delta_theta = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_cms_theta)  # type: str
    pi0_daughters_e = LatexObsNameMethods.add_sub_script(var_str=energy_e, script=LatexParticleNames.gamma_gamma_str)  # type: str
    pi0_daughters_sum_e = r"$\sum " + pi0_daughters_e[1:]  # type: str
    pi0_daughters_diff_e = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_e)  # type: str
    pi0_daughters_diff_e_normed = pi0_daughters_sum_e[:-1] + r" / " + pi0_daughters_diff_e[1:]  # type: str
    pi0_daughters_p = LatexObsNameMethods.add_sub_script(var_str=momentum_p, script=LatexParticleNames.gamma_gamma_str)  # type: str
    pi0_daughters_diff_p = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_p)  # type: str
    pi0_daughters_cms_diff_p = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_diff_p, script=cms_tag)  # type: str
    pi0_daughters_pt = momentum_pt + r"$\,_{" + LatexParticleNames.gamma_gamma_str[1:-1] + "}$"  # type: str
    pi0_daughters_diff_pt = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_pt)  # type: str
    pi0_daughters_cms_diff_pt = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_diff_pt, script=cms_tag)  # type: str
    pi0_daughters_pz = momentum_pz + r"$\,_{" + LatexParticleNames.gamma_gamma_str[1:-1] + "}$"  # type: str
    pi0_daughters_diff_pz = LatexObsNameMethods.get_delta_of(var_str=pi0_daughters_pz)  # type: str
    pi0_daughters_cms_diff_pz = LatexObsNameMethods.add_super_script(var_str=pi0_daughters_diff_pz, script=cms_tag)  # type: str

    k_short_delta_m = LatexObsNameMethods.add_sub_script(var_str=mass_delta_m, script=LatexParticleNames.k_short_str)  # type: str
    k_short_abs_delta_m = LatexObsNameMethods.get_abs_of(var_str=k_short_delta_m)  # type: str

    decay_info_number_of_neutrals_in_event = r"$\#\;\mathrm{Clean\;Neutrals\;in\;Event}$"  # type: str
    decay_info_number_of_tracks_in_event = r"$\#\;\mathrm{Clean\;Tracks\;in\;Event}$"  # type: str
    decay_info_number_of_neutrals_in_b_sig = r"$\#\;\mathrm{Neutrals\;in\;}" + LatexParticleNames.b_sig_str[1:]  # type: str
    decay_info_number_of_tracks_in_b_sig = r"$\#\;\mathrm{Tracks\;in\;}" + LatexParticleNames.b_sig_str[1:]  # type: str
    decay_info_number_of_neutrals_in_b_tag = r"$\#\;\mathrm{Neutrals\;in\;}" + LatexParticleNames.b_tag_str[1:]  # type: str
    decay_info_number_of_tracks_in_b_tag = r"$\#\;\mathrm{Tracks\;in\;}" + LatexParticleNames.b_tag_str[1:]  # type: str

    _ntrls_str = r"$\mathrm{Neutrals}$"  # type: str
    _trks_str = r"$\mathrm{Tracks}$"  # type: str
    _event_str = r"$\mathrm{Event}$"  # type: str
    decay_info_number_of_neutrals = LatexObsNameMethods.add_sub_script(var_str=r"$\mathrm{N}$", script=_ntrls_str)  # type: str
    decay_info_number_of_tracks = LatexObsNameMethods.add_sub_script(var_str=r"$\mathrm{N}$", script=_trks_str)  # type: str

    decay_info_number_of_neutrals_in_event_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_neutrals, script=_event_str)  # type: str
    decay_info_number_of_tracks_in_event_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_tracks, script=_event_str)  # type: str
    decay_info_number_of_neutrals_in_b_sig_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_neutrals, script=LatexParticleNames.b_sig_str)  # type: str
    decay_info_number_of_tracks_in_b_sig_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_tracks, script=LatexParticleNames.b_sig_str)  # type: str
    decay_info_number_of_neutrals_in_b_tag_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_neutrals, script=LatexParticleNames.b_tag_str)  # type: str
    decay_info_number_of_tracks_in_b_tag_short = LatexObsNameMethods.add_super_script(var_str=decay_info_number_of_tracks, script=LatexParticleNames.b_tag_str)  # type: str

    # CS MVA Variables

    cs_base_r2 = r"$\mathrm{R}_{2}$"  # type: str
    cs_base_cos_theta_tbto = r"$\cos\theta_\mathrm{TBTO}$"
    cs_base_cos_theta_tbz = r"$\cos\theta_\mathrm{TBz}$"
    cs_base_thrust_b = r"$\mathrm{T}_\mathrm{B}$"  # type: str
    cs_base_thrust_o = r"$\mathrm{T}_\mathrm{O}$"  # type: str
    cs_base_energy_t = r"$E_t$"  # type: str
    cs_base_mm2 = r"$M_m^2$"  # type: str
    cs_base_h_so_c_0 = r"$H^{so}_{0c}$"  # type: str
    cs_base_h_so_c_1 = r"$H^{so}_{1c}$"  # type: str
    cs_base_h_so_c_2 = r"$H^{so}_{2c}$"  # type: str
    cs_base_h_so_c_3 = r"$H^{so}_{3c}$"  # type: str
    cs_base_h_so_c_4 = r"$H^{so}_{4c}$"  # type: str
    cs_base_h_so_n_0 = r"$H^{so}_{0n}$"  # type: str
    cs_base_h_so_n_2 = r"$H^{so}_{2n}$"  # type: str
    cs_base_h_so_n_4 = r"$H^{so}_{4n}$"  # type: str
    cs_base_h_so_m_0 = r"$H^{so}_{0m}$"  # type: str
    cs_base_h_so_m_2 = r"$H^{so}_{2m}$"  # type: str
    cs_base_h_so_m_4 = r"$H^{so}_{4m}$"  # type: str
    cs_base_h_oo_0 = r"$H^{oo}_{0}$"  # type: str
    cs_base_h_oo_1 = r"$H^{oo}_{1}$"  # type: str
    cs_base_h_oo_2 = r"$H^{oo}_{2}$"  # type: str
    cs_base_h_oo_3 = r"$H^{oo}_{3}$"  # type: str
    cs_base_h_oo_4 = r"$H^{oo}_{4}$"  # type: str
    cs_base_cleo_cone_1 = r"$\mathrm{CleoCone}(1)$"  # type: str
    cs_base_cleo_cone_2 = r"$\mathrm{CleoCone}(2)$"  # type: str
    cs_base_cleo_cone_3 = r"$\mathrm{CleoCone}(3)$"  # type: str
    cs_base_cleo_cone_4 = r"$\mathrm{CleoCone}(4)$"  # type: str
    cs_base_cleo_cone_5 = r"$\mathrm{CleoCone}(5)$"  # type: str
    cs_base_cleo_cone_6 = r"$\mathrm{CleoCone}(6)$"  # type: str
    cs_base_cleo_cone_7 = r"$\mathrm{CleoCone}(7)$"  # type: str
    cs_base_cleo_cone_8 = r"$\mathrm{CleoCone}(8)$"  # type: str
    cs_base_cleo_cone_9 = r"$\mathrm{CleoCone}(9)$"  # type: str

    cs_b_tag_r2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_r2[1:]  # type: str
    cs_b_tag_cos_theta_tbto = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cos_theta_tbto[1:]  # type: str
    cs_b_tag_cos_theta_tbz = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cos_theta_tbz[1:]  # type: str
    cs_b_tag_thrust_b = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_thrust_b[1:]  # type: str
    cs_b_tag_thrust_o = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_thrust_o[1:]  # type: str
    cs_b_tag_energy_t = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_energy_t[1:]  # type: str
    cs_b_tag_mm2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_mm2[1:]  # type: str
    cs_b_tag_h_so_c_0 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_c_0[1:]  # type: str
    cs_b_tag_h_so_c_1 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_c_1[1:]  # type: str
    cs_b_tag_h_so_c_2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_c_2[1:]  # type: str
    cs_b_tag_h_so_c_3 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_c_3[1:]  # type: str
    cs_b_tag_h_so_c_4 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_c_4[1:]  # type: str
    cs_b_tag_h_so_n_0 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_n_0[1:]  # type: str
    cs_b_tag_h_so_n_2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_n_2[1:]  # type: str
    cs_b_tag_h_so_n_4 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_n_4[1:]  # type: str
    cs_b_tag_h_so_m_0 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_m_0[1:]  # type: str
    cs_b_tag_h_so_m_2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_m_2[1:]  # type: str
    cs_b_tag_h_so_m_4 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_so_m_4[1:]  # type: str
    cs_b_tag_h_oo_0 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_oo_0[1:]  # type: str
    cs_b_tag_h_oo_1 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_oo_1[1:]  # type: str
    cs_b_tag_h_oo_2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_oo_2[1:]  # type: str
    cs_b_tag_h_oo_3 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_oo_3[1:]  # type: str
    cs_b_tag_h_oo_4 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_h_oo_4[1:]  # type: str
    cs_b_tag_cleo_cone_1 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_1[1:]  # type: str
    cs_b_tag_cleo_cone_2 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_2[1:]  # type: str
    cs_b_tag_cleo_cone_3 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_3[1:]  # type: str
    cs_b_tag_cleo_cone_4 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_4[1:]  # type: str
    cs_b_tag_cleo_cone_5 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_5[1:]  # type: str
    cs_b_tag_cleo_cone_6 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_6[1:]  # type: str
    cs_b_tag_cleo_cone_7 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_7[1:]  # type: str
    cs_b_tag_cleo_cone_8 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_8[1:]  # type: str
    cs_b_tag_cleo_cone_9 = LatexParticleNames.b_tag_str[:-1] + r"\;" + cs_base_cleo_cone_9[1:]  # type: str

    cs_b_sig_r2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_r2[1:]  # type: str
    cs_b_sig_cos_theta_tbto = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cos_theta_tbto[1:]  # type: str
    cs_b_sig_cos_theta_tbz = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cos_theta_tbz[1:]  # type: str
    cs_b_sig_thrust_b = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_thrust_b[1:]  # type: str
    cs_b_sig_thrust_o = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_thrust_o[1:]  # type: str
    cs_b_sig_energy_t = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_energy_t[1:]  # type: str
    cs_b_sig_mm2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_mm2[1:]  # type: str
    cs_b_sig_h_so_c_0 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_c_0[1:]  # type: str
    cs_b_sig_h_so_c_1 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_c_1[1:]  # type: str
    cs_b_sig_h_so_c_2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_c_2[1:]  # type: str
    cs_b_sig_h_so_c_3 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_c_3[1:]  # type: str
    cs_b_sig_h_so_c_4 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_c_4[1:]  # type: str
    cs_b_sig_h_so_n_0 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_n_0[1:]  # type: str
    cs_b_sig_h_so_n_2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_n_2[1:]  # type: str
    cs_b_sig_h_so_n_4 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_n_4[1:]  # type: str
    cs_b_sig_h_so_m_0 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_m_0[1:]  # type: str
    cs_b_sig_h_so_m_2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_m_2[1:]  # type: str
    cs_b_sig_h_so_m_4 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_so_m_4[1:]  # type: str
    cs_b_sig_h_oo_0 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_oo_0[1:]  # type: str
    cs_b_sig_h_oo_1 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_oo_1[1:]  # type: str
    cs_b_sig_h_oo_2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_oo_2[1:]  # type: str
    cs_b_sig_h_oo_3 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_oo_3[1:]  # type: str
    cs_b_sig_h_oo_4 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_h_oo_4[1:]  # type: str
    cs_b_sig_cleo_cone_1 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_1[1:]  # type: str
    cs_b_sig_cleo_cone_2 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_2[1:]  # type: str
    cs_b_sig_cleo_cone_3 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_3[1:]  # type: str
    cs_b_sig_cleo_cone_4 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_4[1:]  # type: str
    cs_b_sig_cleo_cone_5 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_5[1:]  # type: str
    cs_b_sig_cleo_cone_6 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_6[1:]  # type: str
    cs_b_sig_cleo_cone_7 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_7[1:]  # type: str
    cs_b_sig_cleo_cone_8 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_8[1:]  # type: str
    cs_b_sig_cleo_cone_9 = LatexParticleNames.b_sig_str[:-1] + r"\;" + cs_base_cleo_cone_9[1:]  # type: str

    # fmt: on


if __name__ == "__main__":
    from collections import Counter
    from matplotlib import pyplot as plt

    print("Testing some stuff:")
    latex_labels = [
        LatexObsNames.pid_ksnb_standard,
        LatexObsNames.pid_good_ks,
        LatexObsNames.mva_mm2_sig,
        LatexObsNames.y4s_mass,
        LatexObsNames.q2_reso_v4,
        LatexObsNames.d_st_st_meson_pi0_cms_p,
        LatexObsNames.pi0_good_gamma_0,
        LatexObsNames.pi0_abs_sig_mbf,
        LatexObsNames.pi0_daughters_inv_m,
        LatexObsNames.pi0_daughters_cos_angle,
        LatexObsNames.pi0_daughters_cms_cos_angle,
    ]

    for latex_label in latex_labels:
        print("\t- " + latex_label)

    all_latex_label_names = [
        latex_label_name for latex_label_name in LatexObsNames.__dict__.keys() if not latex_label_name.endswith("__")
    ]  # type: List[str]

    all_var_labels = [
        LatexObsNames.__dict__[latex_label_name] for latex_label_name in all_latex_label_names
    ]  # type: List[str]

    assert len(set(all_var_labels)) == len(all_var_labels), (
        len(set(all_var_labels)),
        len(all_var_labels),
        [v for v, c in Counter(all_var_labels).items() if c > 1],
    )

    test_text = ""
    for latex_label_name in all_latex_label_names:
        latex_label = LatexObsNames.__dict__[latex_label_name]
        print("\t- " + latex_label_name + "  ->  " + latex_label)
        assert latex_label.count("{") == latex_label.count("}")
        assert latex_label.count("$") % 2 == 0, latex_label.count("$")

        test_text += f"{latex_label}\n"

        fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
        try:
            ax.annotate(text=latex_label, xy=(0.5, 0.5), xytext=(0.6, 0.6))
            plt.show()
        except:  # noqa: E722  # Ignore this flake8 error for this test.
            plt.close(fig)
            fig.clf()
            raise ValueError("Error in " + latex_label_name + "  ->  " + latex_label)

        plt.close(fig)
        fig.clf()

    fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
    ax.annotate(text=test_text, xy=(0.5, 0.5), xytext=(0.6, 0.6))
    plt.show()
