"""
Providing latex strings for particles.

Felix Metzner 2021
"""

from rdstar.latex_labels.latex_string_methods import LatexObsNameMethods


__all__ = [
    "LatexParticleNames",
]


class LatexParticleNames:
    others_dots: str = r"$\dots$"

    hadron_str: str = r"$h$"
    hadron_zero_str: str = LatexObsNameMethods.add_super_script(var_str=hadron_str, script=r"$0$")
    hadron_pm_str: str = LatexObsNameMethods.add_super_script(var_str=hadron_str, script=r"$\pm$")
    hadron_plus_str: str = LatexObsNameMethods.add_super_script(var_str=hadron_str, script=r"$+$")
    hadron_minus_str: str = LatexObsNameMethods.add_super_script(var_str=hadron_str, script=r"$-$")
    anti_hadron_zero_str: str = LatexObsNameMethods.get_anti_particle(particle_str=hadron_zero_str)

    hadrons_str = hadron_str[:-1] + others_dots[1:]

    pi_str: str = r"$\pi$"
    wwo_pi_str: str = r"$(\pi)$"
    pi_zero_str: str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$0$")
    pi_pm_str: str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$\pm$")
    pi_plus_str: str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$+$")
    pi_minus_str: str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$-$")

    pi_slow_str: str = LatexObsNameMethods.add_sub_script(var_str=pi_str, script=r"$\mathrm{slow}$")
    pi_pm_slow_str: str = LatexObsNameMethods.add_sub_script(var_str=pi_pm_str, script=r"$\mathrm{slow}$")
    pi_zero_slow_str: str = LatexObsNameMethods.add_sub_script(var_str=pi_zero_str, script=r"$\mathrm{slow}$")

    k_str: str = r"$\mathrm{K}$"
    k_pm_str: str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$\pm$")
    k_mp_str: str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$\mp$")
    k_plus_str: str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$+$")
    k_minus_str: str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$-$")
    k_zero_str: str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$0$")
    k_short_str: str = LatexObsNameMethods.add_sub_script(var_str=k_zero_str, script=r"$\mathrm{Short}$")
    k_long_str: str = LatexObsNameMethods.add_sub_script(var_str=k_zero_str, script=r"$\mathrm{Long}$")

    gamma_str: str = r"$\gamma$"
    gamma_gamma_str: str = gamma_str[:-1] + gamma_str[1:]
    virtual_gamma_str: str = LatexObsNameMethods.add_super_script(var_str=gamma_str, script=r"$\mathrm{Virtual}$")

    electron_str: str = r"$\mathrm{e}$"
    electron_pm_str: str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$\pm$")
    electron_plus_str: str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$+$")
    electron_minus_str: str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$-$")
    mu_str: str = r"$\mu$"
    muon_pm_str: str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$\pm$")
    muon_plus_str: str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$+$")
    muon_minus_str: str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$-$")
    tau_str: str = r"$\tau$"
    tau_pm_str: str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$\pm$")
    tau_plus_str: str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$+$")
    tau_minus_str: str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$-$")
    lepton_str: str = r"$\ell$"
    lepton_pm_str: str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$\pm$")
    lepton_plus_str: str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$+$")
    lepton_minus_str: str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$-$")

    nu_str: str = r"$\nu$"
    nu_lep_str: str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=lepton_str)
    nu_e_str: str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=electron_str)
    nu_mu_str: str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=mu_str)
    nu_tau_str: str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=tau_str)

    anti_nu_str: str = LatexObsNameMethods.get_anti_particle(particle_str=nu_str)
    anti_nu_lep_str: str = LatexObsNameMethods.get_anti_particle(particle_str=nu_lep_str)
    anti_nu_e_str: str = LatexObsNameMethods.get_anti_particle(particle_str=nu_e_str)
    anti_nu_mu_str: str = LatexObsNameMethods.get_anti_particle(particle_str=nu_mu_str)
    anti_nu_tau_str: str = LatexObsNameMethods.get_anti_particle(particle_str=nu_tau_str)

    star_str: str = r"$\ast$"
    star_star_str: str = star_str[:-1] + r"\!" + star_str[1:]
    w_wo_star_str: str = r"$(\ast)$"
    w_wo_star_w_wo_star_str: str = r"$(\ast(\ast))$"

    d_str: str = r"$\mathrm{D}$"
    d_st_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=star_str)
    d_st_st_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=star_star_str)
    d_w_wo_st_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str)
    d_w_wo_st_w_wo_st_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_w_wo_star_str)
    d_w_wo_st_and_pi_str: str = d_w_wo_st_str[:-1] + pi_str[1:]
    d_w_wo_st_and_wwo_pi_str: str = d_w_wo_st_str[:-1] + wwo_pi_str[1:]
    d_st_daughters_str: str = d_st_str[:-1] + r"\;\mathrm{Daughters}$"

    charged_d_w_wo_st_and_pi_str: str = d_w_wo_st_str[:-1] + r"\,\!^{\pm}" + pi_str[1:]
    neutral_d_w_wo_st_and_pi_str: str = d_w_wo_st_str[:-1] + r"\,\!^{0}" + pi_str[1:]

    charged_d_w_wo_st_and_wwo_pi_str: str = d_w_wo_st_str[:-1] + r"\,\!^{\pm}" + wwo_pi_str[1:]
    neutral_d_w_wo_st_and_wwo_pi_str: str = d_w_wo_st_str[:-1] + r"\,\!^{0}" + wwo_pi_str[1:]

    d_st_st_zero_str: str = d_st_st_str[:-1] + r"\,\!^{0}$"
    d_st_st_pm_str: str = d_st_st_str[:-1] + r"\,\!^{\pm}$"
    d_st_st_plus_str: str = d_st_st_str[:-1] + r"\,\!^{+}$"
    d_st_st_minus_str: str = d_st_st_str[:-1] + r"\,\!^{-}$"

    d_w_wo_st_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str[:-1] + r"0$")
    d_w_wo_st_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str[:-1] + r"\pm$")
    d_w_wo_st_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str[:-1] + r"+$")
    d_w_wo_st_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str[:-1] + r"-$")
    d_w_wo_st_zero_and_pi_str: str = d_w_wo_st_zero_str[:-1] + pi_str[1:]
    d_w_wo_st_pm_and_pi_str: str = d_w_wo_st_pm_str[:-1] + pi_str[1:]

    # fmt: off
    d_w_wo_st_w_wo_st_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_w_wo_star_str[:-1] + r"0$")
    d_w_wo_st_w_wo_st_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_w_wo_star_str[:-1] + r"\pm$")
    d_w_wo_st_w_wo_st_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_w_wo_star_str[:-1] + r"+$")
    d_w_wo_st_w_wo_st_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_w_wo_star_str[:-1] + r"-$")
    # fmt: on

    d_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$0$")
    d_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$\pm$")
    d_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$+$")
    d_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$-$")
    d_st_zero_str: str = d_st_str[:-1] + r"\,\!^{0}$"
    d_st_pm_str: str = d_st_str[:-1] + r"\,\!^{\pm}$"
    d_st_plus_str: str = d_st_str[:-1] + r"\,\!^{+}$"
    d_st_minus_str: str = d_st_str[:-1] + r"\,\!^{-}$"

    anti_d_zero_str: str = LatexObsNameMethods.get_anti_particle(particle_str=d_zero_str)
    anti_d_st_zero_str: str = LatexObsNameMethods.get_anti_particle(particle_str=d_st_zero_str)

    d_sszero_st_str: str = LatexObsNameMethods.add_sub_script(var_str=d_st_str, script=r"$0$")
    d_sszero_st_zero_str: str = d_sszero_st_str[:-1] + r"\,\!^{0}$"
    d_sszero_st_pm_str: str = d_sszero_st_str[:-1] + r"\,\!^{\pm}$"
    d_sszero_st_plus_str: str = d_sszero_st_str[:-1] + r"\,\!^{+}$"
    d_sszero_st_minus_str: str = d_sszero_st_str[:-1] + r"\,\!^{-}$"

    d_two_st_str: str = LatexObsNameMethods.add_sub_script(var_str=d_st_str, script=r"$2$")
    d_two_st_zero_str: str = d_two_st_str[:-1] + r"\,\!^{0}$"
    d_two_st_pm_str: str = d_two_st_str[:-1] + r"\,\!^{\pm}$"
    d_two_st_plus_str: str = d_two_st_str[:-1] + r"\,\!^{+}$"
    d_two_st_minus_str: str = d_two_st_str[:-1] + r"\,\!^{-}$"

    d_one_str: str = LatexObsNameMethods.add_sub_script(var_str=d_str, script=r"$1$")
    d_one_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$0$")
    d_one_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$\pm$")
    d_one_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$+$")
    d_one_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$-$")

    d_prime_one_str: str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$\prime$")
    d_prime_one_zero_str: str = d_prime_one_str[:-1] + r"\,\!^{0}$"
    d_prime_one_pm_str: str = d_prime_one_str[:-1] + r"\,\!^{\pm}$"
    d_prime_one_plus_str: str = d_prime_one_str[:-1] + r"\,\!^{+}$"
    d_prime_one_minus_str: str = d_prime_one_str[:-1] + r"\,\!^{-}$"

    d_strange_str: str = LatexObsNameMethods.add_sub_script(var_str=d_str, script=r"$\mathrm{s}$")
    d_strange_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$0$")
    d_strange_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\pm$")
    d_strange_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$+$")
    d_strange_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$-$")

    d_strange_st_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=star_str)
    d_strange_st_zero_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast 0$")
    d_strange_st_pm_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast \pm$")
    d_strange_st_plus_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast +$")
    d_strange_st_minus_str: str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast -$")

    b_str: str = r"$\mathrm{B}$"
    anti_b_str: str = LatexObsNameMethods.get_anti_particle(particle_str=b_str)
    b_zero_str: str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$0$")
    b_pm_str: str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$\pm$")
    b_plus_str: str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$+$")
    b_minus_str: str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$-$")

    sig_str: str = r"$\mathrm{sig}$"
    tag_str: str = r"$\mathrm{tag}$"
    b_sig_str: str = LatexObsNameMethods.add_sub_script(var_str=b_str, script=sig_str)
    b_tag_str: str = LatexObsNameMethods.add_sub_script(var_str=b_str, script=tag_str)

    b_sig_minus_str: str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$-$")
    b_sig_plus_str: str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$+$")
    b_sig_pm_str: str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$\pm$")
    b_sig_neutral_str: str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$0$")
    b_sig_anti_neutral_str: str = LatexObsNameMethods.get_anti_particle(particle_str=b_sig_neutral_str)

    b_tag_minus_str: str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$-$")
    b_tag_plus_str: str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$+$")
    b_tag_pm_str: str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$\pm$")
    b_tag_neutral_str: str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$0$")
    b_tag_anti_neutral_str: str = LatexObsNameMethods.get_anti_particle(particle_str=b_tag_neutral_str)

    y4s_str: str = r"$\Upsilon(4\mathrm{S})$"
    b_b_bar_str = b_str[:-1] + anti_b_str[1:]

    proton_str: str = r"$\mathrm{p}$"
    proton_pm_str: str = LatexObsNameMethods.add_super_script(var_str=proton_str, script=r"$\pm$")

    small_x_str: str = r"$\mathrm{x}$"
    x_str: str = r"$\mathrm{X}$"

    xu_str: str = LatexObsNameMethods.add_sub_script(var_str=x_str, script=r"$\mathrm{u}$")
    xu_zero_str: str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$0$")
    xu_pm_str: str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$\pm$")
    xu_plus_str: str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$+$")
    xu_minus_str: str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$-$")

    xc_str: str = LatexObsNameMethods.add_sub_script(var_str=x_str, script=r"$\mathrm{c}$")
    xc_zero_str: str = LatexObsNameMethods.add_super_script(var_str=xc_str, script=r"$0$")
    xc_pm_str: str = LatexObsNameMethods.add_super_script(var_str=xc_str, script=r"$\pm$")
    xc_plus_str: str = LatexObsNameMethods.add_super_script(var_str=xc_str, script=r"$+$")
    xc_minus_str: str = LatexObsNameMethods.add_super_script(var_str=xc_str, script=r"$-$")

    a_one_str: str = LatexObsNameMethods.add_sub_script(var_str=r"$\mathrm{a}$", script=r"$1$")
    a_one_zero_str: str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$0$")
    a_one_pm_str: str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$\pm$")
    a_one_plus_str: str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$+$")
    a_one_minus_str: str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$-$")

    eta_str: str = r"$\eta$"
    eta_zero_str: str = LatexObsNameMethods.add_super_script(var_str=eta_str, script=r"$0$")

    j_psi_str: str = r"$\mathrm{J}/\psi$"

    # The following strings are just for test purposes:
    __anti_test1_str: str = LatexObsNameMethods.get_anti_particle(particle_str=d_st_zero_str)
    __anti_test2_str: str = LatexObsNameMethods.get_anti_particle(particle_str=d_st_daughters_str)
    __anti_test3_str: str = LatexObsNameMethods.get_anti_particle(particle_str=muon_pm_str)
    __anti_test4_str: str = LatexObsNameMethods.get_anti_particle(particle_str=y4s_str)


if __name__ == "__main__":
    from collections import Counter
    from matplotlib import pyplot as plt

    print("Testing some stuff:")

    all_latex_labels = [label_name for label_name in LatexParticleNames.__dict__.keys() if not label_name.endswith("__")]
    label_to_name = {v: k for k, v in LatexParticleNames.__dict__.items() if not k.endswith("__")}

    all_particle_labels = [LatexParticleNames.__dict__[ln] for ln in all_latex_labels if "__anti_test" not in ln]
    assert len(set(all_particle_labels)) == len(all_particle_labels), (
        len(set(all_particle_labels)),
        len(all_particle_labels),
        [(v, label_to_name[v]) for v, c in Counter(all_particle_labels).items() if c > 1],
    )

    test_text = ""
    for label_name in all_latex_labels:
        particle_label = LatexParticleNames.__dict__[label_name]
        print("\t- " + label_name + "  ->  " + particle_label)
        assert particle_label.count("{") == particle_label.count("}")
        assert particle_label.count("$") % 2 == 0, particle_label.count("$")

        test_text += f"{particle_label}\n"

        fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
        try:
            ax.annotate(text=particle_label, xy=(0.5, 0.5), xytext=(0.6, 0.6))
            plt.show()
        except:  # noqa: E722  # Ignore this flake8 error for this test.
            plt.close(fig)
            fig.clf()
            raise ValueError("Error in " + label_name + "  ->  " + particle_label)

        plt.close(fig)
        fig.clf()

    fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
    ax.annotate(text=test_text, xy=(0.5, 0.5), xytext=(0.6, 0.6))
    plt.show()
