"""
This file contains global variables shared by the B->D**lv signal
extraction analysis.

Max Welsch 2018
"""

from templatefitter.plotter.plot_style import KITColors

from rdstar.offline_analysis.fitting.rdstar_fit_variables import RecoModeNames

# Channel definitions

channel_definitions = {
    # RecoModeNames.DZ: (15, (-1, 3)),
    # RecoModeNames.DM: (15, (-1, 3)),
    # RecoModeNames.DSZ: (15, (-1, 3)),
    # RecoModeNames.DSM: (15, (-1, 3)),
    RecoModeNames.DZPi: (15, (-2.5, 5)),
    RecoModeNames.DMPi: (15, (-2.5, 5)),
    RecoModeNames.DSZPi: (15, (-2.5, 5)),
    RecoModeNames.DSMPi: (10, (-2.5, 5)),
    # RecoModeNames.D: (20, (-1, 5)),
    # RecoModeNames.DS: (20, (-1, 5)),
    # RecoModeNames.DPi: (20, (-2.5, 5)),
    # RecoModeNames.DSPi: (20, (-2.5, 5))
 }

missingmass_mbc_definitions = {
    # RecoModeNames.DZ: (15, (-1, 3)),
    # RecoModeNames.DM: (15, (-1, 3)),
    # RecoModeNames.DSZ: (15, (-1, 3)),
    # RecoModeNames.DSM: (15, (-1, 3)),
    RecoModeNames.DZPi: (20, (-2.5, 7)),
    RecoModeNames.DMPi: (20, (-2.5, 7)),
    RecoModeNames.DSZPi: (20, (-2.5, 7)),
    RecoModeNames.DSMPi: (15, (-2.5, 7)),
    # RecoModeNames.D: (20, (-1, 5)),
    # RecoModeNames.DS: (20, (-1, 5)),
    # RecoModeNames.DPi: (20, (-2.5, 5)),
    # RecoModeNames.DSPi: (20, (-2.5, 5))
 }

neextra_definitions = {
    # RecoModeNames.DZ: (15, (-1, 3)),
    # RecoModeNames.DM: (15, (-1, 3)),
    # RecoModeNames.DSZ: (15, (-1, 3)),
    # RecoModeNames.DSM: (15, (-1, 3)),
    RecoModeNames.DZPi: (40, (0., 3)),
    RecoModeNames.DMPi: (40, (0., 3)),
    RecoModeNames.DSZPi: (40, (0., 3)),
    RecoModeNames.DSMPi: (20, (0., 3)),
    # RecoModeNames.D":(20, (-1, 5)),
    # RecoModeNames.DS: (20, (-1, 5)),
    # RecoModeNames.DPi: (20, (-2.5, 5)),
    # RecoModeNames.DSPi: (20, (-2.5, 5))
 }


channel_defs = {
    "neextra_mbc": neextra_definitions,
    "missingmass_signal": channel_definitions,
    "missingmass_mbc": missingmass_mbc_definitions
}

channel_dec_modes = {
    # RecoModeNames.DM: [11, 12],
    # RecoModeNames.DZ: [13, 14],
    # RecoModeNames.DSM: [15, 16],
    # RecoModeNames.DSZ: [17, 18],
    RecoModeNames.DMPi: [61, 62],
    RecoModeNames.DZPi: [63, 64],
    RecoModeNames.DSMPi: [65, 66],
    RecoModeNames.DSZPi: [67, 68],
    # RecoModeNames.D: [13, 14, 11, 12],
    # RecoModeNames.DS: [17, 18, 15, 16],
    # RecoModeNames.DPi: [63, 64, 61, 62],
    # RecoModeNames.DSPi: [65, 66, 67, 68],

}

channel_to_title = {
    # RecoModeNames.DZ: r"$B^- \rightarrow D^0 \ell^-$",
    # RecoModeNames.DM: r"$B^0 \rightarrow D^- \ell^+$",
    # RecoModeNames.DSZ: r"$B^- \rightarrow D^{*0} \ell^-$",
    # RecoModeNames.DSM: r"$B^0 \rightarrow D^{*-} \ell^+$",
    RecoModeNames.DZPi: r"$B^- \rightarrow D^0 \pi^0 \ell^-$",
    RecoModeNames.DMPi: r"$B^0 \rightarrow D^- \pi^0 \ell^+$",
    RecoModeNames.DSZPi: r"$B^- \rightarrow D^{*0} \pi^0 \ell^-$",
    RecoModeNames.DSMPi: r"$B^0 \rightarrow D^{*-} \pi^0 \ell^+$",
    # RecoModeNames.D: r"$B \rightarrow D \ell$",
    # RecoModeNames.DS: r"$B \rightarrow D^{*} \ell$",
    # RecoModeNames.DPi: r"$B \rightarrow D \pi^0 \ell$",
    # RecoModeNames.DSPi: r"$B \rightarrow D^{*} \pi^0 \ell$",
}

# Template definitions

template_colors = {
    "DSSl": KITColors.kit_red,
    "DSl": KITColors.kit_cyan,
    "Dl": KITColors.kit_blue,
    "Rest": KITColors.kit_green,
}

template_label = {
    "Rest": "Rest",
    "Dl": r"$D \ell \nu$",
    "DSl": r"$D^{*} \ell \nu$",
    "DSSl": r"$D^{**}(\rightarrow D^{(*)} \pi^0) \ell \nu$",
}

template_ids = {
    "Rest": [1, 2, 6, 7, 8, 9, 10],
    "Dl": [3],
    "DSl": [4],
    "DSSl": [5],
}


pid_weights = [
    '__weight_pid_corr_dd_0_ratio__',
    '__weight_pid_corr_dd_1_ratio__',
    '__weight_pid_corr_dd_2_ratio__',
    '__weight_pid_corr_dd_3_ratio__',
    '__weight_pid_corr_dd_4_ratio__',
    '__weight_pid_corr_dst_pi_ratio__',
    '__weight_pid_corr_lepton_ratio__',
]

pid_errors_stat = [
    '__weight_pid_corr_dd_0_stat_uncertainty__',
    '__weight_pid_corr_dd_1_stat_uncertainty__',
    '__weight_pid_corr_dd_2_stat_uncertainty__',
    '__weight_pid_corr_dd_3_stat_uncertainty__',
    '__weight_pid_corr_dd_4_stat_uncertainty__',
    '__weight_pid_corr_dst_pi_stat_uncertainty__',
    '__weight_pid_corr_lepton_stat_uncertainty__',
]

pid_errors_sys = [
    '__weight_pid_corr_dd_0_sys_uncertainty__',
    '__weight_pid_corr_dd_1_sys_uncertainty__',
    '__weight_pid_corr_dd_2_sys_uncertainty__',
    '__weight_pid_corr_dd_3_sys_uncertainty__',
    '__weight_pid_corr_dd_4_sys_uncertainty__',
    '__weight_pid_corr_dst_pi_sys_uncertainty__',
    '__weight_pid_corr_lepton_sys_uncertainty__',
]

pid_weight_uncertainties = [
    '__weight_pid_corr_dd_0_uncertainty_from_var__',
    '__weight_pid_corr_dd_1_uncertainty_from_var__',
    '__weight_pid_corr_dd_2_uncertainty_from_var__',
    '__weight_pid_corr_dd_3_uncertainty_from_var__',
    '__weight_pid_corr_dd_4_uncertainty_from_var__',
    '__weight_pid_corr_dst_pi_uncertainty_from_var__',
    '__weight_pid_corr_lepton_uncertainty_from_var__',
]

done_variations = [
    'weight_ff_Done_var_0_down', 'weight_ff_Done_var_0_up',
    'weight_ff_Done_var_1_down', 'weight_ff_Done_var_1_up',
    'weight_ff_Done_var_2_down', 'weight_ff_Done_var_2_up',
    'weight_ff_Done_var_3_down', 'weight_ff_Done_var_3_up',
]
dtwostar_variations = [
    'weight_ff_Dtwostar_var_0_down', 'weight_ff_Dtwostar_var_0_up',
    'weight_ff_Dtwostar_var_1_down', 'weight_ff_Dtwostar_var_1_up',
    'weight_ff_Dtwostar_var_2_down', 'weight_ff_Dtwostar_var_2_up',
    'weight_ff_Dtwostar_var_3_down', 'weight_ff_Dtwostar_var_3_up',
]
dprimeone_variations = [
    'weight_ff_Dprimeone_var_0_down', 'weight_ff_Dprimeone_var_0_up',
    'weight_ff_Dprimeone_var_1_down', 'weight_ff_Dprimeone_var_1_up',
    'weight_ff_Dprimeone_var_2_down', 'weight_ff_Dprimeone_var_2_up',
]
dzerostar_variations = [
    'weight_ff_Dzerostar_var_0_down', 'weight_ff_Dzerostar_var_0_up',
    'weight_ff_Dzerostar_var_1_down', 'weight_ff_Dzerostar_var_1_up',
    'weight_ff_Dzerostar_var_2_down', 'weight_ff_Dzerostar_var_2_up',
]
btod_variations = [
    'weight_ff_btod_var_0_down', 'weight_ff_btod_var_0_up',
    'weight_ff_btod_var_1_down', 'weight_ff_btod_var_1_up',
    'weight_ff_btod_var_2_down', 'weight_ff_btod_var_2_up',
    'weight_ff_btod_var_3_down', 'weight_ff_btod_var_3_up',
    'weight_ff_btod_var_4_down', 'weight_ff_btod_var_4_up',
]

btodst_variations = [
    'weight_ff_btodst_var_0_down', 'weight_ff_btodst_var_0_up',
    'weight_ff_btodst_var_1_down', 'weight_ff_btodst_var_1_up',
    'weight_ff_btodst_var_2_down', 'weight_ff_btodst_var_2_up',
    'weight_ff_btodst_var_3_down', 'weight_ff_btodst_var_3_up',
    'weight_ff_btodst_var_4_down', 'weight_ff_btodst_var_4_up',
    'weight_ff_btodst_var_5_down', 'weight_ff_btodst_var_5_up',
]

ff_weight_variations = {
    "weight_ff_Done_nom": done_variations,
    "weight_ff_Dtwostar_nom": dtwostar_variations,
    "weight_ff_Dzerostar_nom": dzerostar_variations,
    "weight_ff_Dprimeone_nom": dprimeone_variations,
    "weight_ff_btod_nom": btod_variations,
    "weight_ff_btodst_nom": btodst_variations,
}


def combine_variations(variation_list):
    return list(zip(variation_list[0::2], variation_list[1::2]))


sys_uncertainties = {
    "DSSl": [
        *combine_variations(done_variations),
        *combine_variations(dtwostar_variations),
        *combine_variations(dzerostar_variations),
        *combine_variations(dprimeone_variations),
        ('__weight_pid_corr_dd_0_ratio__var_down', '__weight_pid_corr_dd_0_ratio__var_up'),
        ('__weight_pid_corr_dd_1_ratio__var_down', '__weight_pid_corr_dd_1_ratio__var_up'),
        ('__weight_pid_corr_dd_2_ratio__var_down', '__weight_pid_corr_dd_2_ratio__var_up'),
        ('__weight_pid_corr_dd_3_ratio__var_down', '__weight_pid_corr_dd_3_ratio__var_up'),
        ('__weight_pid_corr_dd_4_ratio__var_down', '__weight_pid_corr_dd_4_ratio__var_up'),
        ('__weight_pid_corr_dst_pi_ratio__var_down', '__weight_pid_corr_dst_pi_ratio__var_up'),
        ('__weight_pid_corr_lepton_ratio__var_down', '__weight_pid_corr_lepton_ratio__var_up'),
    ],
    "DSl": [
        *combine_variations(btodst_variations),
        ('__weight_pid_corr_dd_0_ratio__var_down', '__weight_pid_corr_dd_0_ratio__var_up'),
        ('__weight_pid_corr_dd_1_ratio__var_down', '__weight_pid_corr_dd_1_ratio__var_up'),
        ('__weight_pid_corr_dd_2_ratio__var_down', '__weight_pid_corr_dd_2_ratio__var_up'),
        ('__weight_pid_corr_dd_3_ratio__var_down', '__weight_pid_corr_dd_3_ratio__var_up'),
        ('__weight_pid_corr_dd_4_ratio__var_down', '__weight_pid_corr_dd_4_ratio__var_up'),
        ('__weight_pid_corr_dst_pi_ratio__var_down', '__weight_pid_corr_dst_pi_ratio__var_up'),
        ('__weight_pid_corr_lepton_ratio__var_down', '__weight_pid_corr_lepton_ratio__var_up'),
    ],
    "Dl": [
        *combine_variations(btod_variations),
        ('__weight_pid_corr_dd_0_ratio__var_down', '__weight_pid_corr_dd_0_ratio__var_up'),
        ('__weight_pid_corr_dd_1_ratio__var_down', '__weight_pid_corr_dd_1_ratio__var_up'),
        ('__weight_pid_corr_dd_2_ratio__var_down', '__weight_pid_corr_dd_2_ratio__var_up'),
        ('__weight_pid_corr_dd_3_ratio__var_down', '__weight_pid_corr_dd_3_ratio__var_up'),
        ('__weight_pid_corr_dd_4_ratio__var_down', '__weight_pid_corr_dd_4_ratio__var_up'),
        ('__weight_pid_corr_dst_pi_ratio__var_down', '__weight_pid_corr_dst_pi_ratio__var_up'),
        ('__weight_pid_corr_lepton_ratio__var_down', '__weight_pid_corr_lepton_ratio__var_up'),
    ],
    "Rest": [
        ('__weight_pid_corr_dd_0_ratio__var_down', '__weight_pid_corr_dd_0_ratio__var_up'),
        ('__weight_pid_corr_dd_1_ratio__var_down', '__weight_pid_corr_dd_1_ratio__var_up'),
        ('__weight_pid_corr_dd_2_ratio__var_down', '__weight_pid_corr_dd_2_ratio__var_up'),
        ('__weight_pid_corr_dd_3_ratio__var_down', '__weight_pid_corr_dd_3_ratio__var_up'),
        ('__weight_pid_corr_dd_4_ratio__var_down', '__weight_pid_corr_dd_4_ratio__var_up'),
        ('__weight_pid_corr_dst_pi_ratio__var_down', '__weight_pid_corr_dst_pi_ratio__var_up'),
        ('__weight_pid_corr_lepton_ratio__var_down', '__weight_pid_corr_lepton_ratio__var_up'),
    ],
}

drop_list = [
    'Ecms',
    'InvM',
    'M',
    'Q',
    'ROE_charge__boCleanROE__bc',
    '__weight_pid_corr_dd_0_variation_0__',
    '__weight_pid_corr_dd_0_variation_10__',
    '__weight_pid_corr_dd_0_variation_11__',
    '__weight_pid_corr_dd_0_variation_12__',
    '__weight_pid_corr_dd_0_variation_13__',
    '__weight_pid_corr_dd_0_variation_14__',
    '__weight_pid_corr_dd_0_variation_15__',
    '__weight_pid_corr_dd_0_variation_16__',
    '__weight_pid_corr_dd_0_variation_17__',
    '__weight_pid_corr_dd_0_variation_18__',
    '__weight_pid_corr_dd_0_variation_19__',
    '__weight_pid_corr_dd_0_variation_1__',
    '__weight_pid_corr_dd_0_variation_2__',
    '__weight_pid_corr_dd_0_variation_3__',
    '__weight_pid_corr_dd_0_variation_4__',
    '__weight_pid_corr_dd_0_variation_5__',
    '__weight_pid_corr_dd_0_variation_6__',
    '__weight_pid_corr_dd_0_variation_7__',
    '__weight_pid_corr_dd_0_variation_8__',
    '__weight_pid_corr_dd_0_variation_9__',
    '__weight_pid_corr_dd_1_variation_0__',
    '__weight_pid_corr_dd_1_variation_10__',
    '__weight_pid_corr_dd_1_variation_11__',
    '__weight_pid_corr_dd_1_variation_12__',
    '__weight_pid_corr_dd_1_variation_13__',
    '__weight_pid_corr_dd_1_variation_14__',
    '__weight_pid_corr_dd_1_variation_15__',
    '__weight_pid_corr_dd_1_variation_16__',
    '__weight_pid_corr_dd_1_variation_17__',
    '__weight_pid_corr_dd_1_variation_18__',
    '__weight_pid_corr_dd_1_variation_19__',
    '__weight_pid_corr_dd_1_variation_1__',
    '__weight_pid_corr_dd_1_variation_2__',
    '__weight_pid_corr_dd_1_variation_3__',
    '__weight_pid_corr_dd_1_variation_4__',
    '__weight_pid_corr_dd_1_variation_5__',
    '__weight_pid_corr_dd_1_variation_6__',
    '__weight_pid_corr_dd_1_variation_7__',
    '__weight_pid_corr_dd_1_variation_8__',
    '__weight_pid_corr_dd_1_variation_9__',
    '__weight_pid_corr_dd_2_variation_0__',
    '__weight_pid_corr_dd_2_variation_10__',
    '__weight_pid_corr_dd_2_variation_11__',
    '__weight_pid_corr_dd_2_variation_12__',
    '__weight_pid_corr_dd_2_variation_13__',
    '__weight_pid_corr_dd_2_variation_14__',
    '__weight_pid_corr_dd_2_variation_15__',
    '__weight_pid_corr_dd_2_variation_16__',
    '__weight_pid_corr_dd_2_variation_17__',
    '__weight_pid_corr_dd_2_variation_18__',
    '__weight_pid_corr_dd_2_variation_19__',
    '__weight_pid_corr_dd_2_variation_1__',
    '__weight_pid_corr_dd_2_variation_2__',
    '__weight_pid_corr_dd_2_variation_3__',
    '__weight_pid_corr_dd_2_variation_4__',
    '__weight_pid_corr_dd_2_variation_5__',
    '__weight_pid_corr_dd_2_variation_6__',
    '__weight_pid_corr_dd_2_variation_7__',
    '__weight_pid_corr_dd_2_variation_8__',
    '__weight_pid_corr_dd_2_variation_9__',
    '__weight_pid_corr_dd_3_variation_0__',
    '__weight_pid_corr_dd_3_variation_10__',
    '__weight_pid_corr_dd_3_variation_11__',
    '__weight_pid_corr_dd_3_variation_12__',
    '__weight_pid_corr_dd_3_variation_13__',
    '__weight_pid_corr_dd_3_variation_14__',
    '__weight_pid_corr_dd_3_variation_15__',
    '__weight_pid_corr_dd_3_variation_16__',
    '__weight_pid_corr_dd_3_variation_17__',
    '__weight_pid_corr_dd_3_variation_18__',
    '__weight_pid_corr_dd_3_variation_19__',
    '__weight_pid_corr_dd_3_variation_1__',
    '__weight_pid_corr_dd_3_variation_2__',
    '__weight_pid_corr_dd_3_variation_3__',
    '__weight_pid_corr_dd_3_variation_4__',
    '__weight_pid_corr_dd_3_variation_5__',
    '__weight_pid_corr_dd_3_variation_6__',
    '__weight_pid_corr_dd_3_variation_7__',
    '__weight_pid_corr_dd_3_variation_8__',
    '__weight_pid_corr_dd_3_variation_9__',
    '__weight_pid_corr_dd_4_variation_0__',
    '__weight_pid_corr_dd_4_variation_10__',
    '__weight_pid_corr_dd_4_variation_11__',
    '__weight_pid_corr_dd_4_variation_12__',
    '__weight_pid_corr_dd_4_variation_13__',
    '__weight_pid_corr_dd_4_variation_14__',
    '__weight_pid_corr_dd_4_variation_15__',
    '__weight_pid_corr_dd_4_variation_16__',
    '__weight_pid_corr_dd_4_variation_17__',
    '__weight_pid_corr_dd_4_variation_18__',
    '__weight_pid_corr_dd_4_variation_19__',
    '__weight_pid_corr_dd_4_variation_1__',
    '__weight_pid_corr_dd_4_variation_2__',
    '__weight_pid_corr_dd_4_variation_3__',
    '__weight_pid_corr_dd_4_variation_4__',
    '__weight_pid_corr_dd_4_variation_5__',
    '__weight_pid_corr_dd_4_variation_6__',
    '__weight_pid_corr_dd_4_variation_7__',
    '__weight_pid_corr_dd_4_variation_8__',
    '__weight_pid_corr_dd_4_variation_9__',
    '__weight_pid_corr_dst_pi_variation_0__',
    '__weight_pid_corr_dst_pi_variation_10__',
    '__weight_pid_corr_dst_pi_variation_11__',
    '__weight_pid_corr_dst_pi_variation_12__',
    '__weight_pid_corr_dst_pi_variation_13__',
    '__weight_pid_corr_dst_pi_variation_14__',
    '__weight_pid_corr_dst_pi_variation_15__',
    '__weight_pid_corr_dst_pi_variation_16__',
    '__weight_pid_corr_dst_pi_variation_17__',
    '__weight_pid_corr_dst_pi_variation_18__',
    '__weight_pid_corr_dst_pi_variation_19__',
    '__weight_pid_corr_dst_pi_variation_1__',
    '__weight_pid_corr_dst_pi_variation_2__',
    '__weight_pid_corr_dst_pi_variation_3__',
    '__weight_pid_corr_dst_pi_variation_4__',
    '__weight_pid_corr_dst_pi_variation_5__',
    '__weight_pid_corr_dst_pi_variation_6__',
    '__weight_pid_corr_dst_pi_variation_7__',
    '__weight_pid_corr_dst_pi_variation_8__',
    '__weight_pid_corr_dst_pi_variation_9__',
    '__weight_pid_corr_lepton_variation_0__',
    '__weight_pid_corr_lepton_variation_10__',
    '__weight_pid_corr_lepton_variation_11__',
    '__weight_pid_corr_lepton_variation_12__',
    '__weight_pid_corr_lepton_variation_13__',
    '__weight_pid_corr_lepton_variation_14__',
    '__weight_pid_corr_lepton_variation_15__',
    '__weight_pid_corr_lepton_variation_16__',
    '__weight_pid_corr_lepton_variation_17__',
    '__weight_pid_corr_lepton_variation_18__',
    '__weight_pid_corr_lepton_variation_19__',
    '__weight_pid_corr_lepton_variation_1__',
    '__weight_pid_corr_lepton_variation_2__',
    '__weight_pid_corr_lepton_variation_3__',
    '__weight_pid_corr_lepton_variation_4__',
    '__weight_pid_corr_lepton_variation_5__',
    '__weight_pid_corr_lepton_variation_6__',
    '__weight_pid_corr_lepton_variation_7__',
    '__weight_pid_corr_lepton_variation_8__',
    '__weight_pid_corr_lepton_variation_9__',
    'angleWithMissingMomentum__bo1__cm0__bc',
    'angleWithMissingMomentum__bo1__cm1__bc',
    'angleWithMissingMomentum__bo1__cm2__bc',
    'belleECLEnergy',
    'chiProb',
    'chi_primeReco',
    'costhetalReco',
    'costhetavReco',
    'dM',
    'dQ',
    'daughter__bo0__cm__spPDG__bc',
    'daughter__bo0__cm__spchiProb__bc',
    'daughter__bo0__cm__spcosThetaBetweenParticleAndTrueB__bc',
    'daughter__bo0__cm__spdaughter__bo0__cm__spM__bc__bc',
    'daughter__bo0__cm__spdaughter__bo0__cm__spchiProb__bc__bc',
    'daughter__bo0__cm__spdaughter__bo0__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo0__cm__spdaughter__bo0__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo0__cm__spdaughter__bo1__cm__spM__bc__bc',
    'daughter__bo0__cm__spdaughter__bo1__cm__spchiProb__bc__bc',
    'daughter__bo0__cm__spdaughter__bo1__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo0__cm__spdaughter__bo1__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo0__cm__spdaughter__bo2__cm__spM__bc__bc',
    'daughter__bo0__cm__spdaughter__bo2__cm__spchiProb__bc__bc',
    'daughter__bo0__cm__spdaughter__bo2__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo0__cm__spdaughter__bo2__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc',
    'daughter__bo0__cm__spextraInfo__bouniqueSignal__bc__bc',
    'daughter__bo0__cm__spisExtendedSignal__bc',
    'daughter__bo0__cm__spisPrimarySignal__bc',
    'daughter__bo0__cm__spisSignalAcceptMissingNeutrino__bc',
    'daughter__bo0__cm__spisSignal__bc',
    'daughter__bo0__cm__spmatchedMC__boPDG__bc__bc',
    'daughter__bo0__cm__spmcErrors__bc',
    'daughter__bo0__cm__spnRemainingTracksInEvent__bc',
    'daughter__bo0__cm__spuseCMSFrame__bop__bc__bc',
    'daughter__bo0__cm__spuseCMSFrame__bopt__bc__bc',
    'daughter__bo1__cm__spE__bc',
    'daughter__bo1__cm__spInvM__bc',
    'daughter__bo1__cm__spM__bc',
    'daughter__bo1__cm__spMbc__bc',
    'daughter__bo1__cm__spPDG__bc',
    'daughter__bo1__cm__spQ__bc',
    'daughter__bo1__cm__spROE_charge__boCleanROE__bc__bc',
    'daughter__bo1__cm__spROE_deltae__boCleanROE__bc__bc',
    'daughter__bo1__cm__spROE_eextra__boCleanROE__bc__bc',
    'daughter__bo1__cm__spROE_mbc__boCleanROE__bc__bc',
    'daughter__bo1__cm__spROE_neextra__boCleanROE__bc__bc',
    'daughter__bo1__cm__spchiProb__bc',
    'daughter__bo1__cm__spcosThetaBetweenParticleAndTrueB__bc',
    'daughter__bo1__cm__spdM__bc',
    'daughter__bo1__cm__spdQ__bc',
    'daughter__bo1__cm__spdaughterInvM__bo0__cm__sp2__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spE__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spInvM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spPDG__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spabs__boSigMBF__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spabs__boSigM__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spchiProb__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughterInvariantMass__bo0__cm__sp1__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughterMotherDiffOf__bo0__cm__spM__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spE__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spInvM__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spM__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spQ__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spabs__boSigMBF__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spabs__boSigM__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spchiProb__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdM__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdQ__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spE__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spp__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spE__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spp__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo2__cm__spE__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo2__cm__spp__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo3__cm__spE__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spdaughter__bo3__cm__spp__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spisSignal__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__sppt__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spuseCMSFrame__bop__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo0__cm__spuseCMSFrame__bopt__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spE__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spdaughter__bo1__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spextraInfo__bodecayModeID__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spisSignal__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spp__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__sppt__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__sptheta__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo0__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spE__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spInvM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spchiProb__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spdM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spdQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spisSignal__bc__bc',
    # 'daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcDaughter__bo0__cm__spmcDaughter__bo2__cm__spPDG__bc__bc__bc__bc__bc',
    # 'daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomcDaughter__bo0__cm__spmcDaughter__bo2__cm__spPDG__bc__bc__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomcMother__bomdstIndex__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spmcMother__bomdstIndex__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spnCDCHits__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spnSVDHits__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sppValue__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spp__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sppt__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sppx__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sppy__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sppz__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sptheta__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__sptrackCurlerCloneIndicator__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo1__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spInvM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spPDG__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spabs__boSigMBF__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spchiProb__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdM__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdQ__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughterAngle__bo0__cm1__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughterInvariantMass__bo0__cm__sp1__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughter__bo0__cm__spclusterE9E21__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughter__bo0__cm__spgoodBelleGamma__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughter__bo1__cm__spclusterE9E21__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spdaughter__bo1__cm__spgoodBelleGamma__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spformula__bodaughterDiffOf__bo0__cm__sp1__cm__spE__bc__sp__sl__spdaughterSumOf__boE__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spisSignal__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spp__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spuseCMSFrame__bodaughterAngle__bo0__cm1__bc__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spuseCMSFrame__bop__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spuseCMSFrame__bopt__bc__bc__bc',
    'daughter__bo1__cm__spdaughter__bo2__cm__spuseRestFrame__bodaughterAngle__bo0__cm1__bc__bc__bc__bc',
    'daughter__bo1__cm__spdecayAngle__bo0__bc__bc',
    'daughter__bo1__cm__spdecayAngle__bo1__bc__bc',
    'daughter__bo1__cm__spdeltaE__bc',
    'daughter__bo1__cm__spextraInfo__bolooseMCMotherPDG__bc__bc',
    'daughter__bo1__cm__spextraInfo__bolooseMCWrongDaughterBiB__bc__bc',
    'daughter__bo1__cm__spextraInfo__bolooseMCWrongDaughterN__bc__bc',
    'daughter__bo1__cm__spextraInfo__bolooseMCWrongDaughterPDG__bc__bc',
    'daughter__bo1__cm__spisSignalAcceptMissingNeutrino__bc',
    'daughter__bo1__cm__spmatchedMC__boE__bc__bc',
    'daughter__bo1__cm__spmatchedMC__boPDG__bc__bc',
    'daughter__bo1__cm__spmatchedMC__bopx__bc__bc',
    'daughter__bo1__cm__spmcErrors__bc',
    'daughter__bo1__cm__spnROE_Tracks__boCleanROE__bc__bc',
    'daughter__bo1__cm__sppx__bc',
    'daughter__bo1__cm__sppy__bc',
    'daughter__bo1__cm__sppz__bc',
    'daughter__bo1__cm__spthrustOm__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterAngleInBetween__bo0__cm__sp1__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterAngleInBetween__bo0__cm__sp2__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterAngle__bo0__cm__sp1__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterAngle__bo0__cm__sp2__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterDiffOfPhi__bo0__cm__sp1__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughterDiffOfPhi__bo0__cm__sp2__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughter__bo0__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughter__bo1__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bodaughter__bo2__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bop__bc__bc',
    'daughter__bo1__cm__spuseCMSFrame__bopt__bc__bc',
    'daughter__bo1__cm__spuseRestFrame__bodaughter__bo0__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spuseRestFrame__bodaughter__bo1__cm__spp__bc__bc__bc',
    'daughter__bo1__cm__spuseRestFrame__bodaughter__bo2__cm__spp__bc__bc__bc',
    'extraInfo__boDecayHashExtended__bc',
    'extraInfo__boDecayHash__bc',
    'extraInfo__bodecayModeID__bc',
    'genUpsilon4S__boPDG__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spE__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spPDG__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spcharge__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spchi_primeMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spcosthetalMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spcosthetavMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo2__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo3__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmcDaughter__bo4__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spmdstIndex__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__sppx__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__sppy__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__sppz__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spqSquaredMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo0__cm__spwMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spE__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spPDG__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spcharge__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spchi_primeMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spcosthetalMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spcosthetavMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmmcDaughter__bo2__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo0__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo0__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo1__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmmcDaughter__bo2__cmPDG__bc__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo1__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo2__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo3__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmE__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmPDG__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmpx__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmpy__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmcDaughter__bo4__cmpz__bc__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spmdstIndex__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__sppx__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__sppy__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__sppz__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spqSquaredMC__bc__bc',
    'genUpsilon4S__bomcDaughter__bo1__cm__spwMC__bc__bc',
    'isContinuumEvent',
    'isExtendedSignal',
    'isNotContinuumEvent',
    'isPrimarySignal',
    'isSignal',
    'isSignalAcceptMissingNeutrino',
    'mcErrors',
    'mcMissingMomentum',
    'mcMissingMomentumInBSigRestFrame',
    'missingMomentum',
    'missingMomentumInBSigRestFrame',
    'missingMomentumInCMSFrame',
    'nCleanedECLClusters__bogoodBelleGamma__sp__eq__eq__sp1__spand__spclusterTrackMatch__sp__eq__eq__sp0__bc',
    'nCleanedTracks__bodr__sp__st__sp2__spand__spabs__bodz__bc__sp__st__sp4__bc',
    # 'nCleanedTracks__bodr__sp__st__sp2__spand__spabs__bodz__bc__sp__st__sp4__spand__sppt__sp__gt__sp0__pt2__bc',
    'nECLClusters',
    'nROE_Tracks__boCleanROE__bc',
    'productionIdentifier',
    'qSquaredReco',
    'qSquaredReco2',
    'qSquaredReco3',
    'qSquaredReco4',
    'qSquaredRecoDStarStar',
    'totalEnergyOfParticlesInList__bogamma__clfsp__bc',
    'useCMSFrame__bop__bc',
    'useCMSFrame__bopt__bc',
    'wReco',
]
