"""
Fit observable infos for fit setups of Continuum calibration fit.

Felix Metzner 2023
"""

import numpy as np

from typing import Tuple

from rdstar.utilities.logging import log_to_default

from rdstar.offline_analysis.histogram_variables import RDStarHistVars

from rdstar.offline_analysis.fitting.dedicated_fit_approach.fit_info_container import FitObservableInfo


__all__ = [
    "ContCalObsSetup1",
    "ContCalObsSetup2",
    "ContCalObsSetup3",
    "ContCalBChargeObsSetup1",
]


logger = log_to_default(name=__name__)


class ContCalObsSetup1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.cs_cos_tbto_cs_sideband_var,
        alt_scope=None,
        alt_binning=None,
    )

    cont_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    cont_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)


class ContCalObsSetup2:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.cs_cos_tbz_cs_sideband_var,
        alt_scope=None,
        alt_binning=None,
    )

    cont_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    cont_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)


class ContCalObsSetup3:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.cs_mva_output,
        alt_scope=(0.5, 0.9),
        alt_binning=tuple(np.linspace(0.5, 0.9, 20)),
    )

    cont_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    cont_obs_info_Bz_to_Dm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=None),)
    cont_obs_info_Bm_to_Dz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=None),)
    cont_obs_info_Bz_to_Dsm: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=None),)
    cont_obs_info_Bm_to_Dsz: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=None),)


# Charged and neutral B reco Channels (2 Channels only!)
class ContCalBChargeObsSetup1:
    obs1 = FitObservableInfo(
        hist_var=RDStarHistVars.cs_cos_tbto_cs_sideband_var,
        alt_scope=None,
        alt_binning=None,
    )

    cont_cal_obs_infos: Tuple[FitObservableInfo, ...] = (obs1,)

    cont_obs_info_neutral_b: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
    cont_obs_info_charged_b: Tuple[FitObservableInfo, ...] = (obs1.reset(scope=None, binning=20),)
