import os
import pandas as pd

from typing import Dict
from uncertainties import ufloat

from rdstar.offline_analysis.fitting.dedicated_fit_approach.tag_cal_evaluation.eval_tools import TagCalEvalResult


__all__ = [
    "apply_fei_calibration",
    "apply_full_fei_calibration",
    "apply_bkg_only_fei_calibration",
    "get_rdstar_fei_correction_uncertainties_dict_for_main_tag_cal",
    "apply_stage_one_fei_calibration",
    "apply_stage_one_full_fei_calibration",
    "apply_stage_one_bkg_only_fei_calibration",
    "get_rdstar_stage_one_fei_correction_uncertainties_dict_for_main_tag_cal",
]


_tcr_fit_scenario_name = "tc_main"
_tcr_selection_name = "main_with_mva_selection_sec_sel_mva_with_right_sig_tag_comb"
_tcr_fit_obs_setup_tag = "TagCalObsSetup2p2_mms"

base_tag_cal_res_dir_path = "/home/fmetzner/Downloads/"
tc_eval_name = "TagCalEvaluation_23rdAugust2023_floatingBkg"

_tcr_file_name: str = f"{_tcr_fit_scenario_name}__{_tcr_selection_name}__{_tcr_fit_obs_setup_tag}.json"
_tcr_file_path = os.path.join(base_tag_cal_res_dir_path, tc_eval_name, _tcr_file_name)
assert os.path.isfile(_tcr_file_path), _tcr_file_path


def apply_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_correction(
        df=df,
        apply_for_continuum=False,
    )


def apply_full_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_full_correction(
        df=df,
        apply_for_continuum=False,
    )


def apply_bkg_only_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_simple_correction_for_bkg_only(
        df=df,
        apply_for_continuum=False,
    )


def get_rdstar_fei_correction_uncertainties_dict_for_main_tag_cal() -> Dict[str, ufloat]:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_tcr_file_path)
    return tag_cal_corr_result.get_rdstar_fei_correction_uncertainties_dict()


# Stage One Results

_s1_tcr_fit_scenario_name = "tc_main"
_s1_tcr_selection_name = "main_with_mva_selection_sec_sel_mva_with_right_sig_tag_comb"
_s1_tcr_fit_obs_setup_tag = "TagCalObsSetup2p2_mms"

stage_one_tc_eval_name = "TagCalEvaluation_01stSeptember2023_withFixedBkg_StepOne"

_s1_tcr_file_name: str = f"{_s1_tcr_fit_scenario_name}__{_s1_tcr_selection_name}__{_s1_tcr_fit_obs_setup_tag}.json"
_s1_tcr_file_path = os.path.join(base_tag_cal_res_dir_path, stage_one_tc_eval_name, _s1_tcr_file_name)
assert os.path.isfile(_s1_tcr_file_path), _s1_tcr_file_path


def apply_stage_one_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_s1_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_correction(
        df=df,
        apply_for_continuum=False,
    )


def apply_stage_one_full_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_s1_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_full_correction(
        df=df,
        apply_for_continuum=False,
    )


def apply_stage_one_bkg_only_fei_calibration(df: pd.DataFrame) -> pd.DataFrame:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_s1_tcr_file_path)
    return tag_cal_corr_result.get_mc_sample_with_simple_correction_for_bkg_only(
        df=df,
        apply_for_continuum=False,
    )


def get_rdstar_stage_one_fei_correction_uncertainties_dict_for_main_tag_cal() -> Dict[str, ufloat]:
    tag_cal_corr_result = TagCalEvalResult.load_from_file(file_path=_s1_tcr_file_path)
    return tag_cal_corr_result.get_rdstar_fei_correction_uncertainties_dict()
