"""
Interface to B2BII mDST Input
Felix Metzner 2020
"""

import os
import basf2
import b2biiConversion

from typing import Union, Iterable
from rdstar.utilities import PathType

__all__ = [
    "b2bii_read_and_convert",
]


def b2bii_read_and_convert(
    path: basf2.Path,
    input_files: Union[PathType, Iterable[PathType]],
    input_sequences=None,
    smear_mc_tracks: bool = True,
    smear_mc_track_momenta: bool = True,
) -> None:
    # Information about track smearing:
    # https://belle.kek.jp/secured/wiki/doku.php?id=physics:charm:tracksmearing
    # https://kds.kek.jp/event/6971/contributions/142292/attachments/111232/131990/track_smear.pdf

    b2biiConversion.convertBelleMdstToBelleIIMdst(
        inputBelleMDSTFile=input_files,
        applyHadronBJSkim=True,
        useBelleDBServer=os.environ["BELLE_POSTGRES_SERVER"],
        path=path,
        entrySequences=input_sequences,
        smearTrack=smear_mc_tracks,
        smearTrackMomentum=smear_mc_track_momenta,
    )
